% Induction motor running under pulsating loads
% abc stationary reference frame
% phase variable model
% SI units
 clc,close all
   info= ...
        ['      INDUCTION MOTOR RUNNING UNDER PULSED LOADS                '
         '               [Phase variable model]                           '
         '               (abc stationary reference frame)                 '
         '                      [SI units]                                '
         ' The three-phase motor is running in steady-state at a specified'
         ' load when a pulsating torque load is applied. The initial      '                                      
         ' transients are blanked out.                                    '
         ' The dynamic equations are expressed in canonical first-order   '
         ' matrix form in the abc phase variables in which the state      '
         ' variables are the stator currents ias, ibs, ics; the rotor     '
         ' currents iar, ibr, icr; the rotor speed wm; and the angular    '
         ' displacement theta between stator as_axis and rotor ar_axis.   ' 
         ' The torque, speed,and current responses are plotted.           '
         ' To view the initial transients produced before reaching steady ' 
         ' state, run the file <<IMabcrun1.m>>                            '
         ' To run the identical case study in a Simulink rendition, access'
         ' the model file <<IMabcsim0.mdl>> or press <ABC model>.         '
         ' To examine and compare the identical operation of the motor now'
         ' modeled in dq variables and space vectors, run the model file  '
         ' named <<IMdqrun0.mdl>> or press <DQ model>.                    '];
     
        h0=msgbox(info,'Induction machine transients','help');
        set(h0,'Units','normalized','Resize','on','Position',[.32 .58 .36 .38]) 
              
 % --------------------     MACHINE PARAMETERS [SI] -----------------
Vll=220    ;    %Line-to-line rms voltage rating [V]
Rs=0.531    ;   %Stator resistance (Ohm)
Rr=0.408   ;    %Rotor resistance  (Ohm)
Lsl=2.5e-3 ;    %Stator leakage inductance(H)
Lrl=2.5e-3    ; %Rotor leakage inductance  (H)
Lm=84.7e-3   ;  %Magnetizing inductance  (H) 
freq0=60   ;    %Base frequency (Hz)
wo=2*pi*freq0 ; %Base frequency (rad/s)
p=4        ;    %number of poles
J=0.02    ;     %Moment of inertia (kg.m^2)
TLi=10;         %Initial load torque [N.m]
TLf=2;          %Final load torque [N.m]
Bm=.01;         %Frictional coefficient
% Rotor-stator turns ratio = unity
Vs=Vll*sqrt(2/3);
  waitbar(0,'Computing...');    
%  ---------------------------  ODE  -------------------------------------------
 xo=[0 0 0 0 0 0 wo 0]';         % approximate initial conditions 
 tf=8.0 ;                        % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-3,'maxstep',1e-3); 
 [t,x]=ode45(@IMabcf0,tspan,xo,options,Rs,Rr,Lm,Lsl,Lrl,p, Vs ,wo,J,Bm,TLi,TLf);
 
 % --------------------  OUTPUT VARIABLES ----------------------------------
 spd=x(:,7)*60/(p*pi);  % rpm
 ias=x(:,1)  ;  ibs=x(:,2) ; ics=x(:,3);        % Stator currents [A] 
 Is=sqrt(ias.^2+(ibs-ics).^2/3)  ;              % Stator current magnitude [A]
 iar=x(:,4)  ;  ibr=x(:,5) ; icr=x(:,6) ;       % Rotor currents  [A]
  gam=2*pi/3; spdsync=120*freq0/p;              % Synchronous speed [rpm] 
  vas=Vs*cos(wo*t);
  vbs=Vs*cos(wo*t-gam);
  vcs=Vs*cos(wo*t+gam);  % Three-phase sinusoidal stator voltages (Balanced) [V]
   
  % ----------------- Torque computation ------
   lg=length(t);
  for k=1:lg
    h1=waitbar(k/lg);
    theta=x(k,8);               % Angle between stator and rotor (electric) [rad]
    
    i=x(k,1:6);                 % Current vector   [I]
    Lm1=2/3*Lm;                 % Single phase inductance     
    dLsr=Lm1*[-sin(theta)     -sin(theta+gam) -sin(theta-gam)
             -sin(theta-gam)   -sin(theta)    -sin(theta+gam)
             -sin(theta+gam)  -sin(theta-gam)     -sin(theta)];
        
    dLdtheta=[zeros(3) dLsr
              dLsr'  zeros(3)];
        
    Te(k)=p/4*i*dLdtheta*i'  ;     % Electromagnetic torque [N.m]
     if t(k)<1.5, TL(k)=TLi;
        elseif t(k)>=1.5 & t(k)<5, TL(k)=TLf;  
        elseif t(k)>5, TL(k)=TLi;    
     end                           % Load torque [N.m]
   end
     close(h0,h1)
    
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
  sg=0.02;              % Blanking factor
  rg=(fix(sg*lg):lg);   % Plotting range
  spdrg=ones(1,length(t(rg)))*spdsync;
  figure('Units','Normalized','Position',[.01 .49 .45 .35],'Name','Induction motor transients (abc)');
  figure(1)
  plot(t(rg),spd(rg),'color',[0.5 0 0]);title('Speed vs time');xlabel('Time [s]'),ylabel('Speed [rpm]'),grid
  set(gca,'Xlim',[sg*tf tf]), hold on
  plot(t(rg),spdrg,'-r');legend('motor speed', 'synchronous speed',3)
   figure('Units','Normalized','Position',[.48 .52 .47 .35],'Name','Induction motor transients (abc)');
   figure(2);
   plot(t(rg),vas(rg),'-m');title('Voltage vs time','color','b');xlabel('Time [s]'),ylabel('Voltage [V]'),grid 
   set(gca,'Xlim',[sg*tf tf])
   figure('Units','Normalized','Position',[.01 .08 .45 .32],'Name','Induction motor transients (abc)');
  figure(3) 
   plot(t(rg),ias(rg),'-b',t(rg),ibs(rg),'-g',t(rg),ics(rg),'r',t(rg),Is(rg),'k');title('Currents vs time','color','r');xlabel('Time [s]'),ylabel('Stator currents [A]'),grid
   set(gca,'Xlim',[sg*tf tf])
    figure('Units','Normalized','Position',[.46 .08 .48 .32],'Name','Induction motor transients (abc)'); 
    figure(4) 
   plot(t(rg),iar(rg),'-r',t(rg),ibr(rg),'-g',t(rg),icr(rg),'b');title('Currents vs time','color','r');xlabel('Time [s]'),ylabel('Rotor currents [A]'),grid 
   set(gca,'Xlim',[sg*tf tf])
   figure('Units','Normalized','Position',[.50 .49 .47 .32],'Name','Induction motor transients (abc)');
   figure(5)
    plot(spd(rg),Te(rg),'color',[.6 .2 .4],'linewidth',2);title('Te vs speed','color','r');xlabel('Speed [rpm]'),ylabel('Torque [N.m]'),grid,hold on
    %pt=fix(.1*lg); aa=[spd(pt) Te(pt)]; bb=[spd(pt+2) Te(pt+2)];  vecarrow(aa,bb,'r')
   %pt=fix(.9*lg); aa=[spd(pt) Te(pt)]; bb=[spd(pt+2) Te(pt+2)];  vecarrow(aa,bb,'r')
   figure('Units','Normalized','Position',[.46 .49 .47 .35],'Name','Induction motor transients (abc)');
   figure(6)
   plot(t(rg),Te(rg),'-b',t(rg),TL(rg),'-c');title('Te vs time','color','r');xlabel('Time [s]'),ylabel('Torque [N.m]'),grid
   set(gca,'Xlim',[sg*tf tf]); legend('Te', 'TL',4)
   
   h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.8 .4 .19 .07]); 
       
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit imabcf0', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.07 0.42 0.4 0.2], ...
	      'String','STATE', ...
	      'TooltipString','State');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','run imabcrun0', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.07 0.10 0.4 0.2], ...
	      'String','RUN', ...
	      'TooltipString','Run');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit imabcrun0', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.07 0.75 0.4 0.2], ...
	      'String','PARAM', ...
	      'TooltipString','Param');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','imdqrun0', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.55 0.75 0.4 0.2], ...
	      'String','DQ model', ...
	      'TooltipString','dq');  
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','imabcsim0', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.55 0.42 0.4 0.2], ...
	      'String','ABC model', ...
	      'TooltipString','abc'); 
      
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
          