% Induction motor running under pulsating loads
% abc stationary reference frame
% Currents as state variables
% SI units
 clc,close all
   info= ...
        ['       INDUCTION MOTOR RUNNING UNDER PULSED LOADS              '                                
         '             (abc stationary reference frame)                  '
         '                        [SI units]                             '
         ' The three-phase motor is running at a specified load. After   ' 
         ' reaching steady state in an initialization first run, the     '
         ' initial conditions are established to then proceed to a second'                                        
         ' run in which the load is pulsating.                           '
         ' The dynamic equations are expressed in canonical first-order  ' 
         ' matrix form in the abc stationary frame in which the state    '  
         ' variables are the stator currents ias, ibs, ics; the rotor    ' 
         ' currents iar, ibr, icr; the rotor speed wm; and the angular   '                       
         ' displacement theta between the stator as_axis and the rotor   '                               
         ' direct axis.The solutions are obtained using ODE45.           '                          
         ' The torque, speed,and current responses are plotted.          '
         ' To avoid carrying out the initialization process, run the file'                                                      
         ' <<IMabcrun0>>                                                 '];
     
        h0=msgbox(info,'Induction machine transients','help');
        set(h0,'Units','normalized','Resize','on','Position',[.32 .58 .36 .32]) 
                
 % --------------------     MACHINE PARAMETERS [SI] -----------------
Vll=220    ;    %Line-to-line rms voltage rating [V]
Rs=0.531    ;   %Stator resistance (Ohm)
Rr=0.408   ;    %Rotor resistance  (Ohm)
Lsl=2.5e-3 ;    %Stator leakage inductance(H)
Lrl=2.5e-3    ; %Rotor leakage inductance  (H)
Lm=84.7e-3   ;  %Magnetizing inductance  (H) 
freq0=60   ;    %Base frequency (Hz)
wo=2*pi*freq0 ; %Base frequency (rad/s)
p=4        ;    %number of poles
J=0.02    ;     %Moment of inertia (kg.m^2)
TLi=10;         %Initial load torque [N.m]
TLf=2;          %Final load torque [N.m]
Bm=.001;         %Frictional coefficient
% Rotor-stator turns ratio = unity
Lm1=2/3*Lm;     %Single-phase inductance [H]
Vs=Vll*sqrt(2/3);

% -------------------------  MODE SELECTION  ----------------------------
      kr=menu('Select operating mode','INITIALIZATION','RUN','QUIT');
      if kr>2, return; end
      
%  ---------------------------  ODE  -------------------------------------------
 if kr==1, xo=[0 0 0 0 0 0 wo 0]';         %initial conditions at start
   elseif kr==2, xo=x(length(t),1:8)';     %initial conditions for rerun
 end  
 waitbar(0,'Computing...');
 tf=2.0 ;                                  % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-3,'maxstep',1e-3); 
 [t,x]=ode45(@IMabcf1,tspan,xo,options,Rs,Rr,Lm,Lsl,Lrl,p, Vs ,wo,kr, J,Bm,TLi,TLf);
 
 % --------------------  OUTPUT VARIABLES ----------------------------------
 spd=x(:,7)*60/(p*pi);  % rpm
 ias=x(:,1)  ;  ibs=x(:,2) ; ics=x(:,3);        % Stator currents [A]
 Is=sqrt(ias.^2+(ibs-ics).^2/3)  ;              % Stator current magnitude [A]
 iar=x(:,4)  ;  ibr=x(:,5) ; icr=x(:,6) ;       % Rotor currents  [A]
  gam=2*pi/3;
  vas=Vs*cos(wo*t); vbs=Vs*cos(wo*t-gam);
  vcs=Vs*cos(wo*t+gam);  % Three-phase sinusoidal stator voltages (Balanced) [V]
  % ----------------- Torque computation ------
   lg=length(spd);
   %Te=zeros(l:1g);
   Lm1=2/3*Lm;                  % Single-phase inductance [H]
 for k=1:lg
    h1=waitbar(k/lg);   
    theta=x(k,8);               % Angle between stator and rotor (electric) [rad]
    
    i=x(k,1:6);                 % Current vector   [I]
        
    dLsr=Lm1*[-sin(theta)     -sin(theta+gam) -sin(theta-gam)
            -sin(theta-gam)   -sin(theta)    -sin(theta+gam)
            -sin(theta+gam)  -sin(theta-gam)     -sin(theta)];
        
    dLdtheta=[zeros(3) dLsr
              dLsr'  zeros(3)];
        
    Te(k)=p/4*i*dLdtheta*i'  ;  % Electromagnetic torque [N.m]
 end
    close(h1)
  
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
    figure('Units','Normalized','Position',[.01 .49 .45 .35],'Name','Induction motor transients');
  figure(1)
   plot(t,spd,'color',[0.5 0 0]);title('Speed vs time','color','r');xlabel('Time [s]'),ylabel('Speed [rpm]'),grid
    figure('Units','Normalized','Position',[.48 .52 .47 .35],'Name','Induction motor transients');
   figure(2);
   plot(t,vas,'-m');title('Voltage vs time','color','b');xlabel('Time [s]'),ylabel('Voltage [V]'),grid 
   figure('Units','Normalized','Position',[.01 .08 .45 .32],'Name','Induction motor transients');
  figure(3) 
   plot(t,ias,'-r',t,ibs,'-g',t,ics,'b',t,Is,'k');title('Currents vs time','color','r');xlabel('Time [s]'),ylabel('Stator currents [A]'),grid
    figure('Units','Normalized','Position',[.46 .08 .48 .32],'Name','Induction motor transients'); 
    figure(4) 
   plot(t,iar,'-r',t,ibr,'-g',t,icr,'b');title('Currents vs time','color','r');xlabel('Time [s]'),ylabel('Rotor currents [A]'),grid 
   figure('Units','Normalized','Position',[.50 .49 .47 .32],'Name','Induction motor transients');
   figure(5)
   plot(spd,Te,'-b');title('Te vs speed','color','r');xlabel('Speed [rpm]'),ylabel('Torque [N.m]'),grid,hold on
    pt=fix(.225*lg); aa=[spd(pt) Te(pt)]; bb=[spd(pt+2) Te(pt+2)];  vecarrow(aa,bb,'r')
    pt=fix(.52*lg); aa=[spd(pt) Te(pt)]; bb=[spd(pt+2) Te(pt+2)];  vecarrow(aa,bb,'r'),hold off
   figure('Units','Normalized','Position',[.46 .49 .47 .35],'Name','Induction motor transients');
   figure(6)
   plot(t,Te,'-b');title('Te vs time','color','r');xlabel('Time [s]'),ylabel('Torque [N.m]'),grid
            
  h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.88 .3 .04 .17]); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','clc,close all,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.06 0.55 0.15], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit imabcf1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.59 0.55 0.15], ...
	      'String','STATE', ...
	      'TooltipString','State');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','run imabcrun1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.31 0.55 0.15], ...
	      'String','RUN', ...
	      'TooltipString','Run');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit imabcrun1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.83 0.55 0.15], ...
	      'String','PARAM', ...
	      'TooltipString','Param');

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
