% Synchronous motor start
% Phase variable model... salient pole rotor
% per unit
 clc,close all,clear
   info= ...
        ['                  SYNCHRONOUS MOTOR START                       '
         '                   SALIENT POLE ROTOR                           '
         '                   [Phase variable model]                       '
         '                         [per unit]                             '
         ' A salient pole synchronous motor is started from standstill    '
         ' at rated voltage with the dc field shorted across an external  ' 
         ' field discharge resistor. When synchronous speed is almost     ' 
         ' reached, the discharge resistor is removed and a dc voltage is ' 
         ' applied to the field circuit so that the motor falls into sync.'
         ' A load torque is then suddenly applied.                        '
         ' The dynamic equations are expressed in canonical first-order   ' 
         ' matrix form in the stationary phase variable model in which the'
         ' state variables are the stator currents ias, ibs, ics;the rotor' 
         ' field current iff; the d-axis and q-axis damper currents idk   '
         ' and iqk; the angular speed wm; and the displacement angle theta'
         ' between stator as_axis and rotor direct axis.                  '
         ' The torque, speed,and current responses are plotted.           '];
                                      
      h0=msgbox(info,'Synchronous motor start','help');
      set(h0,'Units','normalized','Resize','on','Position',[.32 .58 .35 .34])  
      
 % --------------------     MACHINE PARAMETERS [pu] -----------------
Ra=0.018     ; %Stator resistance 
Ld=1.15     ; %Direct-axis inductance%%%%%%%%%%%%%%%%%%
Lq=0.90     ; %Quadrature-axis inductance%%%%%%%%%%%%%%
Lal=0.15    ; %Armature leakage inductance
Rfp=0.015   ; %Field resistance (proper)
Rfd=0.185   ; %Field discharge resistor
Lf=1.2      ; %Field inductance
Rkd=0.2     ; %D-axis damper resistance
Rkq=0.2     ; %Q-axis damper resistance
Lkd=1.32    ; %D-axis damper inductance
Lkq=1.26    ; %Q-axis damper inductance
freq0=60    ; %Frequency [Hz]
wo=2*pi*freq0 ; %Base frequency (rad/s)
H=1.0       ; %Inertia constant [s]
TLi=0.2     ; %Initial load torque
TLf=1.0     ; %Final load torque 
Bm=1e-6     ; %Frictional coefficient
Vm=1.0      ; %Stator voltage
t1=4.0      ; %Time of dc field application [s]
t2=8.0      ; %Time of load application [s]
Bm=1e-6     ; %Frictional coefficient
gam=2*pi/3  ;      

waitbar(0,'Computing...');    
%  ---------------------------  ODE  -------------------------------------------
 xo=[0 0 0 0 0 0 0 0]';         % initial conditions 
 tf=10.0;                       % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-2,'maxstep',1e-3); 
 [t,x]=ode45(@SMabcf9,tspan,xo,options,Ra,Rfp,Rkd,Rkq,Ld,Lq,Lal,Lf,Lkd,Lkq,Vm,wo,H,Bm,TLi,TLf,Rfd,t1,t2);
 
   % --------------------  OUTPUT VARIABLES ----------------------------------
 spd=x(:,7);                                    % speed
 ias=x(:,1)  ;  ibs=x(:,2) ; ics=x(:,3);        % Stator currents 
 iff=x(:,4)  ;  ikd=x(:,5) ; ikq=x(:,6) ;       % Rotor currents  
 
  l=length(spd); 
  Lad=Ld-Lal;   Laq=Lq-Lal;
   L2=(Lad-Laq)/2;
  for k=1:l
    h1=waitbar(k/l); 
    theta=x(k,8);               % Angle between stator as_axis and rotor direct axis 
    
    i=x(k,1:6);                 % Current vector
   Lfkd=1.0; Laf=1.0; Lakd=1.0; Lakq=1.0;   
   
    dLss=-2*L2*[sin(2*theta)       sin(2*theta-gam)  sin(2*theta+gam)  
                  sin(2*theta-gam) sin(2*(theta-gam))    sin(2*theta)
                  sin(2*theta+gam)   sin(2*theta)    sin(2*(theta+gam))];  
         
    
    dLsr=[-Laf*sin(theta)     -Lakd*sin(theta)      -Lakq*cos(theta)
          -Laf*sin(theta-gam) -Lakd*sin(theta-gam)  -Lakq*cos(theta-gam)
          -Laf*sin(theta+gam) -Lakq*sin(theta+gam)  -Lakq*cos(theta+gam)];
            
    dLdtheta=[dLss   dLsr
              dLsr'  zeros(3)];
        
    Te(k)=0.5*i*dLdtheta*i'  ;  % Electromagnetic torque  
    
 end
%  close(h0,h1)
  
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
   figure('Units','Normalized','Position',[.01 .49 .45 .35],'Name','Synchronous motor transients');
  figure(1)
  plot(t,spd,'color',[0.5 0 0]);title('Synchronous motor start','color','r');xlabel('Time [s]'),ylabel('Speed [rpm]'),grid
  
  figure('Units','Normalized','Position',[.46 .49 .45 .35],'Name','Synchronous motor starting transients');
  figure(2)
    plot(t,Te,'-m',[0 t2],[TLi TLi],'-c',[t2 tf],[TLf TLf],'-c',[0 tf],[0 0],'-k');title('Synchronous motor start','color','r');
           xlabel('Time [s]'),ylabel('Torque [pu]'),grid
           legend('Te', 'TL')
           
  figure('Units','Normalized','Position',[.01 .08 .45 .32],'Name','Synchronous motor start');
  figure(3) 
   plot(t,ias,'-r',t,ibs,'-g',t,ics,'b');title('Synchronous motor start','color','r');xlabel('Time [s]'),ylabel('Stator currents [pu]'),grid
   
  figure('Units','Normalized','Position',[.46 .08 .48 .32],'Name','Synchronous motor transients'); 
    figure(4) 
   plot(t,iff,'-g');title('Synchronous motor start','color','r');xlabel('Time [s]'),ylabel('Field current [pu]'),grid 
   
      h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.88 .50 .05 .1]); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','clc,close all,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.1 0.6 0.22], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','smabcrun9', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.31 0.6 .22], ...
	      'String','RERUN', ...
	      'TooltipString','Run');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcf9', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.55 0.6 .22], ...
	      'String','STATE', ...
	      'TooltipString','State');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcrun9', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.77 .6 0.22], ...
	      'String','PARAM', ...
	      'TooltipString','Parameters');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

